@@ -1,6 +1,6 @@
 import datetime
 from decimal import Decimal
-from itertools import chain
+from unittest import skipUnless
 
 from django.core.exceptions import FieldDoesNotExist, FieldError
 from django.db import connection
@@ -14,10 +14,10 @@ from django.db.models import (
     Exists,
     ExpressionWrapper,
     F,
-    FilteredRelation,
     FloatField,
     Func,
     IntegerField,
+    JSONField,
     Max,
     OuterRef,
     Q,
@@ -36,6 +36,7 @@ from django.db.models.functions import (
     Lower,
     Trim,
 )
+from django.db.models.sql.query import get_field_names_from_opts
 from django.test import TestCase, skipUnlessDBFeature
 from django.test.utils import register_lookup
 
@@ -45,6 +46,7 @@ from .models import (
     Company,
     DepartmentStore,
     Employee,
+    JsonModel,
     Publisher,
     Store,
     Ticket,
@@ -468,6 +470,16 @@ class NonAggregateAnnotationTestCase(TestCase):
                 )
             )
 
+    def test_values_wrong_annotation(self):
+        expected_message = (
+            "Cannot resolve keyword 'annotation_typo' into field. Choices are: %s"
+        )
+        article_fields = ", ".join(
+            ["annotation"] + sorted(get_field_names_from_opts(Book._meta))
+        )
+        with self.assertRaisesMessage(FieldError, expected_message % article_fields):
+            Book.objects.annotate(annotation=Value(1)).values_list("annotation_typo")
+
     def test_decimal_annotation(self):
         salary = Decimal(10) ** -Employee._meta.get_field("salary").decimal_places
         Employee.objects.create(
@@ -560,6 +572,16 @@ class NonAggregateAnnotationTestCase(TestCase):
         self.assertEqual(book["other_rating"], 4)
         self.assertEqual(book["other_isbn"], "155860191")
 
+    def test_values_fields_annotations_order(self):
+        qs = Book.objects.annotate(other_rating=F("rating") - 1).values(
+            "other_rating", "rating"
+        )
+        book = qs.get(pk=self.b1.pk)
+        self.assertEqual(
+            list(book.items()),
+            [("other_rating", self.b1.rating - 1), ("rating", self.b1.rating)],
+        )
+
     def test_values_with_pk_annotation(self):
         # annotate references a field in values() with pk
         publishers = Publisher.objects.values("id", "book__rating").annotate(
@@ -947,6 +969,24 @@ class NonAggregateAnnotationTestCase(TestCase):
         ):
             Book.objects.annotate(BooleanField(), Value(False), is_book=True)
 
+    def test_complex_annotations_must_have_an_alias(self):
+        complex_annotations = [
+            F("rating") * F("price"),
+            Value("title"),
+            Case(When(pages__gte=400, then=Value("Long")), default=Value("Short")),
+            Subquery(
+                Book.objects.filter(publisher_id=OuterRef("pk"))
+                .order_by("-pubdate")
+                .values("name")[:1]
+            ),
+            Exists(Book.objects.filter(publisher_id=OuterRef("pk"))),
+        ]
+        msg = "Complex annotations require an alias"
+        for annotation in complex_annotations:
+            with self.subTest(annotation=annotation):
+                with self.assertRaisesMessage(TypeError, msg):
+                    Book.objects.annotate(annotation)
+
     def test_chaining_annotation_filter_with_m2m(self):
         qs = (
             Author.objects.filter(
@@ -1116,32 +1156,13 @@ class NonAggregateAnnotationTestCase(TestCase):
         )
 
     def test_alias_sql_injection(self):
+        crafted_alias = """injected_name" from "annotations_book"; --"""
         msg = (
-            "Column aliases cannot contain whitespace characters, hashes, "
-            "control characters, quotation marks, semicolons, or SQL comments."
+            "Column aliases cannot contain whitespace characters, quotation marks, "
+            "semicolons, or SQL comments."
         )
-        for crafted_alias in [
-            """injected_name" from "annotations_book"; --""",
-            # Control characters.
-            *(f"name{chr(c)}" for c in chain(range(32), range(0x7F, 0xA0))),
-        ]:
-            with self.subTest(crafted_alias):
-                with self.assertRaisesMessage(ValueError, msg):
-                    Book.objects.annotate(**{crafted_alias: Value(1)})
-
-    def test_alias_filtered_relation_sql_injection(self):
-        msg = (
-            "Column aliases cannot contain whitespace characters, hashes, "
-            "control characters, quotation marks, semicolons, or SQL comments."
-        )
-        for crafted_alias in [
-            """injected_name" from "annotations_book"; --""",
-            # Control characters.
-            *(f"name{chr(c)}" for c in chain(range(32), range(0x7F, 0xA0))),
-        ]:
-            with self.subTest(crafted_alias):
-                with self.assertRaisesMessage(ValueError, msg):
-                    Book.objects.annotate(**{crafted_alias: FilteredRelation("author")})
+        with self.assertRaisesMessage(ValueError, msg):
+            Book.objects.annotate(**{crafted_alias: Value(1)})
 
     def test_alias_forbidden_chars(self):
         tests = [
@@ -1155,25 +1176,35 @@ class NonAggregateAnnotationTestCase(TestCase):
             "ali/*as",
             "alias*/",
             "alias;",
-            # [] and # are used by MSSQL.
+            # [] are used by MSSQL.
             "alias[",
             "alias]",
-            "ali#as",
-            "ali\0as",
         ]
         msg = (
-            "Column aliases cannot contain whitespace characters, hashes, "
-            "control characters, quotation marks, semicolons, or SQL comments."
+            "Column aliases cannot contain whitespace characters, quotation marks, "
+            "semicolons, or SQL comments."
         )
         for crafted_alias in tests:
             with self.subTest(crafted_alias):
                 with self.assertRaisesMessage(ValueError, msg):
                     Book.objects.annotate(**{crafted_alias: Value(1)})
 
-                with self.assertRaisesMessage(ValueError, msg):
-                    Book.objects.annotate(
-                        **{crafted_alias: FilteredRelation("authors")}
-                    )
+    @skipUnless(connection.vendor == "postgresql", "PostgreSQL tests")
+    @skipUnlessDBFeature("supports_json_field")
+    def test_set_returning_functions(self):
+        class JSONBPathQuery(Func):
+            function = "jsonb_path_query"
+            output_field = JSONField()
+            set_returning = True
+
+        test_model = JsonModel.objects.create(
+            data={"key": [{"id": 1, "name": "test1"}, {"id": 2, "name": "test2"}]}, id=1
+        )
+        qs = JsonModel.objects.annotate(
+            table_element=JSONBPathQuery("data", Value("$.key[*]"))
+        ).filter(pk=test_model.pk)
+
+        self.assertEqual(qs.count(), len(qs))
 
 
 class AliasTests(TestCase):
@@ -1439,41 +1470,15 @@ class AliasTests(TestCase):
                 with self.assertRaisesMessage(FieldError, msg):
                     getattr(qs, operation)("rating_alias")
 
+    def test_alias_after_values(self):
+        qs = Book.objects.values_list("pk").alias(other_pk=F("pk"))
+        self.assertEqual(qs.get(pk=self.b1.pk), (self.b1.pk,))
+
     def test_alias_sql_injection(self):
+        crafted_alias = """injected_name" from "annotations_book"; --"""
         msg = (
-            "Column aliases cannot contain whitespace characters, hashes, "
-            "control characters, quotation marks, semicolons, or SQL comments."
+            "Column aliases cannot contain whitespace characters, quotation marks, "
+            "semicolons, or SQL comments."
         )
-        for crafted_alias in [
-            """injected_name" from "annotations_book"; --""",
-            # Control characters.
-            *(f"name{chr(c)}" for c in chain(range(32), range(0x7F, 0xA0))),
-        ]:
-            with self.subTest(crafted_alias):
-                with self.assertRaisesMessage(ValueError, msg):
-                    Book.objects.alias(**{crafted_alias: Value(1)})
-
-    def test_alias_filtered_relation_sql_injection(self):
-        msg = (
-            "Column aliases cannot contain whitespace characters, hashes, "
-            "control characters, quotation marks, semicolons, or SQL comments."
-        )
-        for crafted_alias in [
-            """injected_name" from "annotations_book"; --""",
-            # Control characters.
-            *(f"name{chr(c)}" for c in chain(range(32), range(0x7F, 0xA0))),
-        ]:
-            with self.subTest(crafted_alias):
-                with self.assertRaisesMessage(ValueError, msg):
-                    Book.objects.alias(**{crafted_alias: FilteredRelation("authors")})
-
-    def test_alias_filtered_relation_sql_injection_dollar_sign(self):
-        qs = Book.objects.alias(
-            **{"crafted_alia$": FilteredRelation("authors")}
-        ).values("name", "crafted_alia$")
-        if connection.vendor == "postgresql":
-            msg = "Dollar signs are not permitted in column aliases on PostgreSQL."
-            with self.assertRaisesMessage(ValueError, msg):
-                list(qs)
-        else:
-            self.assertEqual(qs.first()["name"], self.b1.name)
+        with self.assertRaisesMessage(ValueError, msg):
+            Book.objects.alias(**{crafted_alias: Value(1)})
