@@ -1,7 +1,7 @@
 import logging
 from contextlib import contextmanager
 from io import StringIO
-from unittest import TestCase
+from unittest import TestCase, mock
 
 from admin_scripts.tests import AdminScriptTestCase
 
@@ -94,6 +94,7 @@ class DefaultLoggingTests(
 
 
 class LoggingAssertionMixin:
+
     def assertLogRecord(
         self,
         logger_cm,
@@ -509,10 +510,30 @@ class AdminEmailHandlerTest(SimpleTestCase):
         msg = mail.outbox[0]
         self.assertEqual(msg.subject, "[Django] ERROR: message")
         self.assertEqual(len(msg.alternatives), 1)
-        body_html = str(msg.alternatives[0][0])
+        body_html = str(msg.alternatives[0].content)
         self.assertIn('<div id="traceback">', body_html)
         self.assertNotIn("<form", body_html)
 
+    @override_settings(ADMINS=[])
+    def test_emit_no_admins(self):
+        handler = AdminEmailHandler()
+        record = self.logger.makeRecord(
+            "name",
+            logging.ERROR,
+            "function",
+            "lno",
+            "message",
+            None,
+            None,
+        )
+        with mock.patch.object(
+            handler,
+            "format_subject",
+            side_effect=AssertionError("Should not be called"),
+        ):
+            handler.emit(record)
+        self.assertEqual(len(mail.outbox), 0)
+
 
 class SettingsConfigTest(AdminScriptTestCase):
     """
@@ -631,20 +652,18 @@ args=(sys.stdout,)
 [formatter_simple]
 format=%(message)s
 """
-        self.temp_file = NamedTemporaryFile()
-        self.temp_file.write(logging_conf.encode())
-        self.temp_file.flush()
+        temp_file = NamedTemporaryFile()
+        temp_file.write(logging_conf.encode())
+        temp_file.flush()
+        self.addCleanup(temp_file.close)
         self.write_settings(
             "settings.py",
             sdict={
                 "LOGGING_CONFIG": '"logging.config.fileConfig"',
-                "LOGGING": 'r"%s"' % self.temp_file.name,
+                "LOGGING": 'r"%s"' % temp_file.name,
             },
         )
 
-    def tearDown(self):
-        self.temp_file.close()
-
     def test_custom_logging(self):
         out, err = self.run_manage(["check"])
         self.assertNoOutput(err)
@@ -701,6 +720,7 @@ class LogFormattersTests(SimpleTestCase):
 
 
 class LogResponseRealLoggerTests(LoggingAssertionMixin, TestCase):
+
     request = RequestFactory().get("/test-path/")
 
     def test_missing_response_raises_attribute_error(self):
@@ -846,17 +866,19 @@ class LogResponseRealLoggerTests(LoggingAssertionMixin, TestCase):
 
         msg = "Test message: %s"
         for case, expected in test_cases:
-            with self.assertLogs("django.request", level="ERROR") as cm:
-                with self.subTest(case=case):
-                    response = HttpResponse(status=318)
-                    log_response(msg, case, response=response, level="error")
+            with (
+                self.assertLogs("django.request", level="ERROR") as cm,
+                self.subTest(case=case),
+            ):
+                response = HttpResponse(status=318)
+                log_response(msg, case, response=response, level="error")
 
-                    record = self.assertLogRecord(
-                        cm,
-                        msg % expected,
-                        levelno=logging.ERROR,
-                        status_code=318,
-                        request=None,
-                    )
-                    # Log record is always a single line.
-                    self.assertEqual(len(record.getMessage().splitlines()), 1)
+                record = self.assertLogRecord(
+                    cm,
+                    msg % expected,
+                    levelno=logging.ERROR,
+                    status_code=318,
+                    request=None,
+                )
+                # Log record is always a single line.
+                self.assertEqual(len(record.getMessage().splitlines()), 1)
