@@ -9,11 +9,13 @@ from django.test.utils import CaptureQueriesContext, isolate_apps
 from .models import (
     Base,
     Chef,
+    CommonChild,
     CommonInfo,
     CustomSupplier,
     GrandChild,
     GrandParent,
     ItalianRestaurant,
+    ItalianRestaurantCommonParent,
     MixinModel,
     Parent,
     ParkingLot,
@@ -149,6 +151,36 @@ class ModelInheritanceTests(TestCase):
         # accidentally found).
         self.assertSequenceEqual(s.titles.all(), [])
 
+    def test_create_diamond_mti_default_pk(self):
+        # 1 INSERT for each base.
+        with self.assertNumQueries(4):
+            common_child = CommonChild.objects.create()
+        # 3 SELECTs for the parents, 1 UPDATE for the child.
+        with self.assertNumQueries(4):
+            common_child.save()
+
+    def test_create_diamond_mti_common_parent(self):
+        with self.assertNumQueries(4):
+            italian_restaurant_child = ItalianRestaurantCommonParent.objects.create(
+                name="Ristorante Miron",
+                address="1234 W. Ash",
+            )
+
+        self.assertEqual(
+            italian_restaurant_child.italianrestaurant_ptr.place_ptr,
+            italian_restaurant_child.place_ptr_two,
+        )
+        self.assertEqual(
+            italian_restaurant_child.italianrestaurant_ptr.restaurant_ptr,
+            italian_restaurant_child.restaurant_ptr,
+        )
+        self.assertEqual(
+            italian_restaurant_child.restaurant_ptr.place_ptr,
+            italian_restaurant_child.place_ptr_two,
+        )
+        self.assertEqual(italian_restaurant_child.name, "Ristorante Miron")
+        self.assertEqual(italian_restaurant_child.address, "1234 W. Ash")
+
     def test_update_parent_filtering(self):
         """
         Updating a field of a model subclass doesn't issue an UPDATE
@@ -184,9 +216,11 @@ class ModelInheritanceTests(TestCase):
             GrandChild().save()
 
         for i, test in enumerate([a, b]):
-            with self.subTest(i=i), self.assertNumQueries(4), CaptureQueriesContext(
-                connection
-            ) as queries:
+            with (
+                self.subTest(i=i),
+                self.assertNumQueries(4),
+                CaptureQueriesContext(connection) as queries,
+            ):
                 test()
                 for query in queries:
                     sql = query["sql"]
@@ -309,6 +343,11 @@ class ModelInheritanceTests(TestCase):
 
         self.assertEqual(type(MethodOverride.foo), DeferredAttribute)
 
+    def test_full_clean(self):
+        restaurant = Restaurant.objects.create()
+        with self.assertNumQueries(0), self.assertRaises(ValidationError):
+            restaurant.full_clean()
+
 
 class ModelInheritanceDataTests(TestCase):
     @classmethod
