@@ -124,6 +124,11 @@ class TestFindStatic(TestDefaults, CollectionTestCase):
             searched_locations,
         )
 
+    def test_missing_args_message(self):
+        msg = "Enter at least one staticfile."
+        with self.assertRaisesMessage(CommandError, msg):
+            call_command("findstatic")
+
 
 class TestConfiguration(StaticFilesTestCase):
     def test_location_empty(self):
@@ -142,11 +147,12 @@ class TestConfiguration(StaticFilesTestCase):
             storage.staticfiles_storage._wrapped = empty
             with self.settings(
                 STORAGES={
+                    **settings.STORAGES,
                     STATICFILES_STORAGE_ALIAS: {
                         "BACKEND": (
                             "django.contrib.staticfiles.storage.StaticFilesStorage"
                         )
-                    }
+                    },
                 }
             ):
                 command = collectstatic.Command()
@@ -155,9 +161,10 @@ class TestConfiguration(StaticFilesTestCase):
             storage.staticfiles_storage._wrapped = empty
             with self.settings(
                 STORAGES={
+                    **settings.STORAGES,
                     STATICFILES_STORAGE_ALIAS: {
                         "BACKEND": "staticfiles_tests.storage.DummyStorage"
-                    }
+                    },
                 }
             ):
                 command = collectstatic.Command()
@@ -250,6 +257,7 @@ class TestCollectionVerbosity(CollectionTestCase):
 
     @override_settings(
         STORAGES={
+            **settings.STORAGES,
             STATICFILES_STORAGE_ALIAS: {
                 "BACKEND": (
                     "django.contrib.staticfiles.storage.ManifestStaticFilesStorage"
@@ -264,6 +272,7 @@ class TestCollectionVerbosity(CollectionTestCase):
 
     @override_settings(
         STORAGES={
+            **settings.STORAGES,
             STATICFILES_STORAGE_ALIAS: {
                 "BACKEND": (
                     "django.contrib.staticfiles.storage.ManifestStaticFilesStorage"
@@ -297,6 +306,7 @@ class TestCollectionClear(CollectionTestCase):
 
     @override_settings(
         STORAGES={
+            **settings.STORAGES,
             STATICFILES_STORAGE_ALIAS: {
                 "BACKEND": "staticfiles_tests.storage.PathNotImplementedStorage"
             },
@@ -416,6 +426,7 @@ class TestCollectionDryRun(TestNoFilesCreated, CollectionTestCase):
 
 @override_settings(
     STORAGES={
+        **settings.STORAGES,
         STATICFILES_STORAGE_ALIAS: {
             "BACKEND": "django.contrib.staticfiles.storage.ManifestStaticFilesStorage"
         },
@@ -462,18 +473,14 @@ class TestCollectionFilesOverride(CollectionTestCase):
 
         os.utime(self.testfile_path, (self.orig_atime - 1, self.orig_mtime - 1))
 
-        self.settings_with_test_app = self.modify_settings(
+        settings_with_test_app = self.modify_settings(
             INSTALLED_APPS={"prepend": "staticfiles_test_app"},
         )
         with extend_sys_path(self.temp_dir):
-            self.settings_with_test_app.enable()
-
+            settings_with_test_app.enable()
+        self.addCleanup(settings_with_test_app.disable)
         super().setUp()
 
-    def tearDown(self):
-        super().tearDown()
-        self.settings_with_test_app.disable()
-
     def test_ordering_override(self):
         """
         Test if collectstatic takes files in proper order
@@ -544,6 +551,7 @@ class TestCollectionOverwriteWarning(CollectionTestCase):
 
 @override_settings(
     STORAGES={
+        **settings.STORAGES,
         STATICFILES_STORAGE_ALIAS: {
             "BACKEND": "staticfiles_tests.storage.DummyStorage"
         },
@@ -571,6 +579,7 @@ class TestCollectionNonLocalStorage(TestNoFilesCreated, CollectionTestCase):
 class TestCollectionNeverCopyStorage(CollectionTestCase):
     @override_settings(
         STORAGES={
+            **settings.STORAGES,
             STATICFILES_STORAGE_ALIAS: {
                 "BACKEND": "staticfiles_tests.storage.NeverCopyRemoteStorage"
             },
@@ -642,9 +651,10 @@ class TestCollectionLinks(TestDefaults, CollectionTestCase):
 
     @override_settings(
         STORAGES={
+            **settings.STORAGES,
             STATICFILES_STORAGE_ALIAS: {
                 "BACKEND": "staticfiles_tests.storage.PathNotImplementedStorage"
-            }
+            },
         }
     )
     def test_no_remote_link(self):
