@@ -6,7 +6,7 @@ from unittest import mock
 from django.test import SimpleTestCase
 from django.utils.datastructures import MultiValueDict
 from django.utils.http import (
-    _urlsplit,
+    MAX_URL_LENGTH,
     base36_to_int,
     content_disposition_header,
     escape_leading_slashes,
@@ -35,18 +35,7 @@ class URLEncodeTests(SimpleTestCase):
 
     def test_dict(self):
         result = urlencode({"a": 1, "b": 2, "c": 3})
-        # Dictionaries are treated as unordered.
-        self.assertIn(
-            result,
-            [
-                "a=1&b=2&c=3",
-                "a=1&c=3&b=2",
-                "b=2&a=1&c=3",
-                "b=2&c=3&a=1",
-                "c=3&a=1&b=2",
-                "c=3&b=2&a=1",
-            ],
-        )
+        self.assertEqual(result, "a=1&b=2&c=3")
 
     def test_dict_containing_sequence_not_doseq(self):
         self.assertEqual(urlencode({"a": [1, 2]}, doseq=False), "a=%5B1%2C+2%5D")
@@ -80,14 +69,7 @@ class URLEncodeTests(SimpleTestCase):
             ),
             doseq=True,
         )
-        # MultiValueDicts are similarly unordered.
-        self.assertIn(
-            result,
-            [
-                "name=Adrian&name=Simon&position=Developer",
-                "position=Developer&name=Adrian&name=Simon",
-            ],
-        )
+        self.assertEqual(result, "name=Adrian&name=Simon&position=Developer")
 
     def test_dict_with_bytes_values(self):
         self.assertEqual(urlencode({"a": b"abc"}, doseq=True), "a=abc")
@@ -292,45 +274,20 @@ class URLHasAllowedHostAndSchemeTests(unittest.TestCase):
                     False,
                 )
 
-    # TODO: Remove when dropping support for PY38.
-    def test_invalid_bracketed_hosts(self):
-        # Port of urllib.parse.urlsplit() tests from Python.
-        tests = [
-            "Scheme://user@[192.0.2.146]/Path?Query",
-            "Scheme://user@[important.com:8000]/Path?Query",
-            "Scheme://user@[v123r.IP]/Path?Query",
-            "Scheme://user@[v12ae]/Path?Query",
-            "Scheme://user@[v.IP]/Path?Query",
-            "Scheme://user@[v123.]/Path?Query",
-            "Scheme://user@[v]/Path?Query",
-            "Scheme://user@[0439:23af::2309::fae7:1234]/Path?Query",
-            "Scheme://user@[0439:23af:2309::fae7:1234:2342:438e:192.0.2.146]/"
-            "Path?Query",
-            "Scheme://user@]v6a.ip[/Path",
+    def test_max_url_length(self):
+        allowed_host = "example.com"
+        max_extra_characters = "é" * (MAX_URL_LENGTH - len(allowed_host) - 1)
+        max_length_boundary_url = f"{allowed_host}/{max_extra_characters}"
+        cases = [
+            (max_length_boundary_url, True),
+            (max_length_boundary_url + "ú", False),
         ]
-        for invalid_url in tests:
-            with self.subTest(invalid_url=invalid_url):
-                self.assertRaises(ValueError, _urlsplit, invalid_url)
-
-    # TODO: Remove when dropping support for PY38.
-    def test_splitting_bracketed_hosts(self):
-        # Port of urllib.parse.urlsplit() tests from Python.
-        p1 = _urlsplit("scheme://user@[v6a.ip]/path?query")
-        self.assertEqual(p1.hostname, "v6a.ip")
-        self.assertEqual(p1.username, "user")
-        self.assertEqual(p1.path, "/path")
-        # Removed the '%test' suffix from ported tests as %scope_id suffixes were
-        # added in Python 3.9: https://docs.python.org/3/whatsnew/3.9.html#ipaddress
-        p2 = _urlsplit("scheme://user@[0439:23af:2309::fae7]/path?query")
-        self.assertEqual(p2.hostname, "0439:23af:2309::fae7")
-        self.assertEqual(p2.username, "user")
-        self.assertEqual(p2.path, "/path")
-        p3 = _urlsplit(
-            "scheme://user@[0439:23af:2309::fae7:1234:192.0.2.146]/path?query"
-        )
-        self.assertEqual(p3.hostname, "0439:23af:2309::fae7:1234:192.0.2.146")
-        self.assertEqual(p3.username, "user")
-        self.assertEqual(p3.path, "/path")
+        for url, expected in cases:
+            with self.subTest(url=url):
+                self.assertIs(
+                    url_has_allowed_host_and_scheme(url, allowed_hosts={allowed_host}),
+                    expected,
+                )
 
 
 class URLSafeBase64Tests(unittest.TestCase):
@@ -393,10 +350,9 @@ class HttpDateProcessingTests(unittest.TestCase):
         )
 
     @unittest.skipIf(platform.architecture()[0] == "32bit", "The Year 2038 problem.")
-    @mock.patch("django.utils.http.datetime.datetime")
+    @mock.patch("django.utils.http.datetime")
     def test_parsing_rfc850(self, mocked_datetime):
         mocked_datetime.side_effect = datetime
-        mocked_datetime.now = mock.Mock()
         now_1 = datetime(2019, 11, 6, 8, 49, 37, tzinfo=timezone.utc)
         now_2 = datetime(2020, 11, 6, 8, 49, 37, tzinfo=timezone.utc)
         now_3 = datetime(2048, 11, 6, 8, 49, 37, tzinfo=timezone.utc)
@@ -571,6 +527,7 @@ class ContentDispositionHeaderTests(unittest.TestCase):
                 (True, '"espécimen" filename'),
                 "attachment; filename*=utf-8''%22esp%C3%A9cimen%22%20filename",
             ),
+            ((True, "some\nfile"), "attachment; filename*=utf-8''some%0Afile"),
         )
 
         for (is_attachment, filename), expected in tests:
